﻿using System;
using System.Windows.Forms;
using System.Collections.Generic;
using System.Linq;
using Infragistics.Win.UltraWinGrid;


namespace ResizeBindInfragisticsGrid
{
    public partial class Form1 : Form
    {
        //private List<User> users = new List<User>();
        private List<Country> countries;

        private struct Cols
        {
            public const string Nick = "Nick";
            public const string City = "City";
            public const string Country = "Country";
            public const string BirthDate = "BirthDate";
        }

        public Form1()
        {
            InitializeComponent();
            countries = new List<Country>();
            countries.Add(new Country("Moldova"));
            countries.Add(new Country("France"));
            countries.Add(new Country("Australia"));
            countries.Add(new Country("Canada"));

            cmbCountry.DisplayMember = "Name";
            cmbCountry.DataSource = countries;
        }

        private void Form1_Shown(object sender, EventArgs e)
        {
            StartNewUser();
        }

        private void btnAdd_Click(object sender, EventArgs e)
        {
            ControlsToActiveRow();
            StartNewUser();
        }

        private void StartNewUser()
        {
            var r = this.myGrid.DisplayLayout.Bands[0].AddNew();
            this.myGrid.ActiveRow = r;

            txtNick.ResetText();
            cmbCountry.ResetText();
            cmbCity.ResetText();
            txtBirthDate.ResetText();
            cmbCity.DataSource = null;
            cmbCity.Enabled = false;
        }

        private void ActiveRowToControls()
        {
            var r = myGrid.ActiveRow;
            if (r == null) return;

            txtNick.Text = r.Cells[Cols.Nick].OriginalValue.ToString();
            cmbCountry.Value = r.Cells[Cols.Country].OriginalValue;
            cmbCity.Value = r.Cells[Cols.City].OriginalValue;
            txtBirthDate.Text = r.Cells[Cols.BirthDate].OriginalValue.ToString();
        }

        private void ControlsToActiveRow()
        {
            var r = myGrid.ActiveRow;
            if (r == null) return;

            r.Cells[Cols.Nick].Value = txtNick.Text;
            if (cmbCountry.Value == null)
                r.Cells[Cols.Country].Value = null;//System.DBNull.Value; //(object)null;
            else
                r.Cells[Cols.Country].Value = cmbCountry.Value;
            if (cmbCity.Value == null)
                r.Cells[Cols.City].Value = null;//System.DBNull.Value;
            else
                r.Cells[Cols.City].Value = cmbCity.Value;
            r.Cells[Cols.BirthDate].Value = txtBirthDate.Text;
        }

        private void cmbCountry_RowSelected(object sender, Infragistics.Win.UltraWinGrid.RowSelectedEventArgs e)
        {
            var selectedCountry = from c in countries where c.Name == cmbCountry.Text select c;
            if (selectedCountry.Count() > 0)
            {
                var country = selectedCountry.First();

                cmbCountry.Value = country;
                List<string> cities = country.Cities;
                cmbCity.ResetText();
                cmbCity.DataSource = cities;
                cmbCity.Enabled = true;
            }
            ControlsToActiveRow();
        }

        private void textBox_Validated(object sender, EventArgs e)
        {
            ControlsToActiveRow();
        }

        private void winCombo_RowSelected(object sender, Infragistics.Win.UltraWinGrid.RowSelectedEventArgs e)
        {
            ControlsToActiveRow();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            // Build a Dictionary which associates each column in the grid with a control
            // on the the form. 
            Dictionary<UltraGridColumn, Control> relatedControls = new Dictionary<UltraGridColumn, Control>();
            UltraGridBand band = myGrid.DisplayLayout.Bands[0];
            relatedControls[band.Columns[Cols.Nick]] = this.txtNick;
            relatedControls[band.Columns[Cols.Country]] = this.cmbCountry;
            relatedControls[band.Columns[Cols.City]] = this.cmbCity;
            relatedControls[band.Columns[Cols.BirthDate]] = this.txtBirthDate;

            // Pass that Dictionary into the CreationFilter. 
            this.myGrid.CreationFilter = new MyGridCreationFilter(relatedControls);
        }

        private void myGrid_AfterRowActivate(object sender, EventArgs e)
        {
            UltraGridRow row = myGrid.ActiveRow;
            myGrid.Selected.Rows.Clear();
            myGrid.Selected.Rows.Add(row);
            ActiveRowToControls();
        }

        private void btnAnalyzer_Click(object sender, EventArgs e)
        {
            // this loads the object from the grid

            // say, we have to number how many cities there are in the grid's Counties
            // stupid, I know, but just to "see" Objects in cells that represents Texts

            List<Country> countries = new List<Country>();
            // obtain all the countries
            foreach (var row in myGrid.Rows)
            {
                object oCoutry = row.Cells[Cols.Country].Value;
                if (oCoutry != DBNull.Value)
                    countries.Add((Country)oCoutry);
            }

            int countCities = 0;
            int countCountries = 0;
            foreach (var c in countries)
            {
                countCountries++;
                if (c.Cities != null)
                    countCities += c.Cities.Count;
            }

            MessageBox.Show(
                string.Format("There are {0} cities in {1} grid countries!",
                countCities, countCountries));
        }

    }

    //public class User
    //{
    //    public User(string nick)
    //    {
    //        this.Nick = nick;
    //        this.Country = new Country("France");
    //        this.City = this.Country.Cities[0];
    //    }

    //    public string Nick { get; set; }
    //    public Country Country { get; set; }
    //    public string City { get; set; }
    //    public DateTime BirthDate { get; set; }
    //}

    public class Country
    {
        public Country(string name)
        {
            this.Name = name;
            this.Cities = new List<string>() { "City 1 " + name, "City 2 " + name };
        }
        public string Name { get; set; }
        public List<string> Cities;

        public override string ToString()
        {
            return this.Name;
        }
    }
}
